/* 
 * QDMX Client Library
 * 
 * Copyright 2006, 2009 
 * - Julien Etelain < julien at pmad dot net >
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */
#ifndef _QDMX_CLIENT_H_
#define _QDMX_CLIENT_H_

#include<QObject>
#include<QList>
#include<QHostAddress>

#ifndef QDMX_PORT
#define QDMX_PORT 0xFE90
#endif

#ifndef QDMX_MAX_CHANNEL
#define QDMX_MAX_CHANNEL 512
#endif

#ifndef QDMX_LIB_VERSION
#define QDMX_LIB_VERSION 0x00020000
#endif

class QDmxClientUniverse;

class QDmxConnectivity;

class QDmxClientPrivate;
class QDmxClientDriverInfo;

/**@file*/


class QDmxClient : public QObject
{
  Q_OBJECT

  friend class QDmxClientUniverse;

  public:

  QDmxClient(QObject *parent = 0);
  
  ~QDmxClient();

  bool connectAutoLocal();

  bool connectAutoStartLocal();

  void connectAutoStartLocalAsync();
  
  QString qdmxsvExecutable();
  
  QString settingsQdmxsvExecutable();

  void setSettingsQdmxsvExecutable(const QString&);

  bool connectToLocal(const QString&);
  
  bool connectToServer(const QHostAddress& host=QHostAddress::LocalHost, quint16 port=QDMX_PORT );
  
  void disconnectFromServer();
  
  bool isConnected();

  bool fetchUniverses();
  
  bool redetectUniverses();
  
  bool stopServer();
  
  int ping();

  QString aboutServer();
  
  QString versionInfo();

  int universesCount();
  
  QDmxClientUniverse* getUniverse(int);
  
  QDmxClientUniverse* getUniverseByUid(quint32);
  
  QDmxClientUniverse* getUniverseByPermaId(const QString&);
  
	QDmxClientUniverse *configureUniverse ( const QString& permaId, const QString& parameter, bool permanent );
  
	QDmxClientUniverse *getOrConfigureUniverse ( const QString& permaId, const QString& parameter=QString(), bool permanent=false );
	
	const QList<QDmxClientDriverInfo*>& getDriversList();
	
  signals:
  void disconnected();
  
  /**
   * If connectAutoLocalAsync() was called, tell that library is connected
   */
  void connected();
  
  /**
   * If connectAutoLocalAsync() was called, tell that library failed to connect and to start server
   */
  void connectionFailure(bool);
  
  private slots:
  void wasDisconnected();
  
  void asyncLookup();
  
  private:
  QDmxConnectivity *link;
  QDmxClientPrivate *p;

};

class QDmxClientUniversePrivate;

class QDmxClientUniverse : public QObject
{

  Q_OBJECT
  
  friend class QDmxClient;
  friend class QDmxClientChannelListener;
  friend class QDmxClientPrivate;
  
  
  public:
  /** Universe modes */
	enum Mode {
	 /** Universe has no special mode (usually simple buffer) */
	 ModeNone = 0x3,
	 /** Universe emit DMX signal to outside of QDmx */
	 ModeEmit = 0x1,
	 /** Universe receive DMX signal from outside of QDmx */
	 ModeReceive = 0x2
	};

  bool setChannels(int from, int len, unsigned char* values);
  
  bool getChannels(int from, int len, unsigned char* values);

  bool setActiveChannelsCount(unsigned short n);
  
  bool getActiveChannelsCount(unsigned short &n);

  void getListenChannels(int from, int len, unsigned char* values);
  
  void getListenActiveChannelsCount(unsigned short &n);

  bool start();
  
  bool stop();

  bool refreshInformations();

  const QString& getPermaId();
  
  const QString& getName();
  
  const QString& getParameter();

  quint32 getUid();

  qint32 getActiveUsersCount();
  
  bool isStarted();
  
  bool isAvailable();
  
  bool isConfiguredPermanently();
  
  Mode getMode();
  
  bool isReceiveMode();
  
  bool isEmitMode();

  bool startListening();
  
  void stopListening();

  bool isListening();

  bool isConfigureSupported();

	bool reconfigureUniverse ( const QString& permaId, const QString& parameter );

	bool permanentUniverse (  bool permanent );

  signals:
  /**
   * Signal emitted when connection is lost, or server report than universe is no more available
   */
  void notAvailable();
  
  /**
   * Signal emitted when listening has stopped.
   *
   * This can the result of server connection loss, or a call to stopListening().
   */
  void stoppedListening();
  
  /**
   * Is listening, emitted each time a new DMX frame is received, or at least in an interval defined by server.
   *
   * @param length Length of DMX frame (number of active channels)
   * @param data DMX Data
   *
   * @see startListening()
   * @see stopListening()
   * @see isListening()
   */
  void newFrame ( quint16 length, quint8 * data );  
  
  private slots:
  void listenerDisconnected();
  
  void listenerReadyRead();
  
  private:
  QDmxConnectivity *link;  
  
  QDmxConnectivity *listenerLink;
    
  QDmxClientUniversePrivate *p;
  
  QDmxClientUniverse(QDmxClient *);
  
  ~QDmxClientUniverse();
  
  bool processError(int);
  
  void hasLostConnection();

};


class QDmxClientDriverInfoPrivate;


class QDmxClientDriverInfo
{
  friend class QDmxClientPrivate;
  friend class QDmxClient;
  
  public:
  bool isConfigureSupported();
  const QString& getName();
  const QString& getPermaIdPrefix();
  const QString& getAbout();
  
  private:
  QDmxClientDriverInfo();
  ~QDmxClientDriverInfo();
  QDmxClientDriverInfoPrivate *p;
};



#endif
